@@ -7,6 +7,7 @@
 from talos.thrift.auth.ttypes import Credential
 from talos.thrift.auth import constants
 from talos.thrift.transport.TTransport import TTransportBase
+from talos.thrift.common.ttypes import GalaxyTalosException
 from io import BytesIO
 from datetime import datetime
 from six.moves import urllib
@@ -402,7 +403,8 @@ class TalosHttpClient(TTransportBase):
         self.message = self.__http_response.reason
         self._headers = self.__http_response.msg
         if not self.code == 200:
-            pass
+            raise GalaxyTalosException(errorCode=self.code, errMsg=self.message)
+            # pass
         # TODO Trans Exception to TalosTransportError
         # reason = response.fp.read()
         # raise TalosTransportError(self.code, self.message, current_time_mills())
