@@ -16,8 +16,9 @@ from talos.thrift.message.MessageService import GetMessageRequest
 from talos.thrift.message.MessageService import GetMessageResponse
 from atomic import AtomicLong
 from talos.client.compression.Compression import Compression
-from talos.thrift.common.ttypes import GalaxyTalosException
+from talos.thrift.transport.TSocket import TTransportException
 from talos.thrift.message.ttypes import MessageOffset
+from talos.client.ScheduleInfoCache import ScheduleInfoCache
 import logging
 
 
@@ -35,20 +36,25 @@ class SimpleConsumer:
     def __init__(self, clientConfig=None, topicName=None, partitionId=None,
                  topicAndPartition=None, credential=None, messageClient=None):
         self.consumerConfig = clientConfig
+        self.talosClientFactory = TalosClientFactory(clientConfig, credential)
         if topicName:
             Utils.check_topic_name(topicName)
             self.topicName = topicName
             self.partitionId = partitionId
             self.credential = credential
-            talosClientFactory = TalosClientFactory(clientConfig, credential)
-            self.messageClient = talosClientFactory.new_message_client()
-            self.get_topic_info(talosClientFactory.new_topic_client(), topicName,
+            self.messageClient = self.talosClientFactory.new_message_client()
+            self.get_topic_info(self.talosClientFactory.new_topic_client(), topicName,
                                 partitionId)
         else:
             self.messageClient = messageClient
             self.topicAndPartition = topicAndPartition
         self.simpleConsumerId = Utils.generate_client_id(clientConfig.get_client_ip(), "")
         self.requestId = AtomicLong(1)
+        self.scheduleInfoCache = ScheduleInfoCache().get_schedule_info_cache(
+            topicTalosResourceName=self.topicAndPartition.topicTalosResourceName,
+            talosClientConfig=self.consumerConfig, messageClient=self.messageClient,
+            talosClientFactory=self.talosClientFactory)
+        self.isActive = True
 
     def get_topic_info(self, topicClient=None, topicName=None, partitionId=None):
         response = topicClient.get_describe_info(GetDescribeInfoRequest(topicName))
@@ -82,10 +88,24 @@ class SimpleConsumer:
 
         getMessageResponse = GetMessageResponse(messageBlocks=[])
         try:
-            getMessageResponse = self.messageClient.get_message(getMessageRequest)
-        except GalaxyTalosException as e:
-            self.logger.error("fetch message failed! " + e.details)
-            raise e
+            getMessageResponse = self.scheduleInfoCache.get_or_create_message_client(
+                topicAndPartition=self.topicAndPartition).get_message(getMessageRequest)
+        except TTransportException as tTransportException:
+            if not self.scheduleInfoCache and self.scheduleInfoCache.get_is_auto_location():
+                self.logger.warn("can't connect to the host directly, refresh scheduleInfo and retry using url. "
+                                 + "The exception message is :" + tTransportException.message +
+                                 ". Ignore this if not frequently.")
+                self.scheduleInfoCache.update_schedule_info_cache()
+                getMessageResponse = self.messageClient.get_message(getMessageRequest)
+            else:
+                self.logger.error("fetch message failed! ")
+                raise tTransportException
+
+        # update scheduleInfocache when request have been transfered and talos auto location was set up
+        if getMessageResponse.isTransfer is not None and getMessageResponse.isTransfer and self.scheduleInfoCache is not None \
+                and self.scheduleInfoCache.get_is_auto_location():
+            self.logger.info("request has been transfered when talos auto location set up, refresh scheduleInfo")
+            self.scheduleInfoCache.update_schedule_info_cache()
 
         messageAndOffsetList = Compression().decompress(getMessageResponse.messageBlocks,
                                                         getMessageResponse.unHandledMessageNumber)
@@ -104,5 +124,13 @@ class SimpleConsumer:
             end = len(messageAndOffsetList)
             return messageAndOffsetList[start:end]
 
-
+    def shut_down(self):
+        if not self.isActive:
+            self.logger.info("SimpleConsumer which simpleConsumerId is: " + str(self.simpleConsumerId) +
+                             " is already shutdown, don't do it again.")
+            return
+        self.logger.info("consumer of " + self.topicAndPartition.topicTalosResourceName + " is shutting down.")
+        self.scheduleInfoCache.shut_down(self.topicAndPartition.topicTalosResourceName)
+        self.logger.info("consumer of " + self.topicAndPartition.topicTalosResourceName + " shutdown.")
+        self.isActive = False
 
